%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% MCDCOutput.m
%
classdef MCDCOutput < handle
  
  properties (Access = public)
    
    j
    IDX       % J
    SKP       % J (max), xdim, 2 (Sigma, L)
    OKP       % J (max), ydim, 2 (Sigma, L)
    FV        % J (max), xdim, nGrid (x1, x2, ... in PMCMC2)
    GV        % J (max), ydim, nGrid (x1, x2, ... in PMCMC2)
    XE        % J (max), T, xdim
    YE        % J (max), T, ydim
    loglik    % J (max)
    
  end
  
  methods (Access = public)
    
    function this = MCDCOutput(J)
      this.j = 0;
      this.IDX = zeros(J, 1);
      this.SKP = [];
      this.OKP = [];
      this.FV = [];
      this.GV = [];
      this.XE = [];
      this.YE = [];
      this.loglik = [];
    end
    
    function result = SKPAt(this, j)
      result = this.at(this.SKP, j);
    end
    
    function result = OKPAt(this, j)
      result = this.at(this.OKP, j);
    end
    
    function result = FVAt(this, j)
      result = this.at(this.FV, j);
    end
    
    function result = GVAt(this, j)
      result = this.at(this.GV, j);
    end
    
    function result = XEAt(this, j)
      result = this.at(this.XE, j);
    end
    
    function result = YEAt(this, j)
      result = this.at(this.YE, j);
    end
    
    function result = loglikAt(this, j)
      function r = f(i)
        if (1 <= i && i <= this.j)
          r = this.loglik(this.IDX(i));
        else
          r = 0;
        end
      end
      
      result = arrayfun(@f, j');
    end
    
    function SetIDXAsLast(this, i)
      this.IDX(i) = size(this.loglik, 1);
    end
    
    function AddSKP(this, skp)
      this.SKP = cat(1, this.SKP, permute(skp, [3 1 2]));
    end
    
    function AddOKP(this, okp)
      this.OKP = cat(1, this.OKP, permute(okp, [3 1 2]));
    end
    
    function AddFV(this, fv)
      this.FV = cat(1, this.FV, permute(fv, [ndims(fv)+1 1:ndims(fv)]));
    end
    
    function AddGV(this, gv)
      this.GV = cat(1, this.GV, permute(gv, [ndims(gv)+1 1:ndims(gv)]));
    end
    
    function AddXE(this, xe)
      this.XE = cat(1, this.XE, permute(xe, [3 1 2]));
    end
    
    function AddYE(this, ye)
      this.YE = cat(1, this.YE, permute(ye, [3 1 2]));
    end
    
    function AddLoglik(this, loglik)
      this.loglik = cat(1, this.loglik, loglik);
    end
    
  end
  
  methods (Access = private)
  
    function result = at(this, array, j)
      arraySize = size(array);
      function r = f(i)
        if (1 <= i && i <= this.j)
          r = reshape(array(this.IDX(i), :), [1 arraySize(2:end)]);
        else
          r = zeros([1 arraySize(2:end)]);
        end
      end

      result = cell2mat(arrayfun(@f, j', 'UniformOutput', false));
    end

  end

end
